@@ -48,7 +48,7 @@ def test_basic_parse_to_arg():
     with pytest.raises(TypeError, match="missing 10 required keyword arguments"):
         MyTupBasic()
     arg_cmd = 'MyTupBasic+ --a_int 32 --a_float 0.3 --a_bool True --a_str hello --b_list_int 1 2 3 --b_set_str set1 set2 ' + \
-              '--d_tuple_3 10 0.5 False --d_tuple_2 tuple 12 --e_dict_str_int size:32 area:90 --e_dict_int_bool 10:True 20:False 30:True --with_default 10 MyTupBasic-'
+              '--d_tuple_3 10 0.5 False --d_tuple_2 tuple 12 --e_dict_str_int size:32 area:90 --e_dict_int_bool 10:True 20:False 30:True MyTupBasic-'
     parsed_arg = MyTupBasic(arg_cmd.split())
     assert parsed_arg == my_tup_basic
     parsed_arg_from_factory: MyTupBasic = MyTupBasic.__from_argv__(arg_cmd.split())
@@ -61,11 +61,15 @@ def test_basic_parse_to_arg():
     assert my_parser._option_string_actions['--a_str'].choices == ['hello', 'bonjour', 'hola']
 
 
+class Muted:
+    def write(self, *_):
+        pass
+
+
+from contextlib import redirect_stderr
+
+
 def test_parse_error():
-    class Muted:
-        def write(self, *_):
-            pass
-    from contextlib import redirect_stderr
     with redirect_stderr(Muted()):
         pytest.raises(SystemExit, MyTupBasic, [])
 
@@ -75,11 +79,16 @@ def test_optional():
     class MyTup(NamedTuple):
         ints: Optional[List[int]] = None
 
-    pytest.raises(SystemExit, MyTup, ['--ints', 'None'])
+    with redirect_stderr(Muted()):
+        pytest.raises(SystemExit, MyTup, ['--ints', 'None'])
     # assert MyTup(['--ints', 'None']).ints is None
     assert MyTup(['--ints', '1', '2']).ints == [1, 2]
     assert MyTup(['--ints']).ints == []
-    pytest.raises(SmartArgError, arg_suite, NamedTuple('InvalidOptional', no=Optional[int]))
+
+    class InvalidOptional(NamedTuple):
+        no: Optional[int]
+
+    pytest.raises(SmartArgError, arg_suite, InvalidOptional)
 
 
 def test_post_process():
@@ -108,12 +117,14 @@ def test_validate():
     @arg_suite
     class MyTup(NamedTuple):
         a_int: int = LateInit  # if a_int is not in the argument, post_process will initialize it
+
     validated = False
 
     def validate(s):
         nonlocal validated
         validated = True
         raise AttributeError()
+
     MyTup.__validate__ = validate
     pytest.raises(AttributeError, MyTup, ['--a_int', '1'])
     assert validated, "`validate` might not be executed."
@@ -133,6 +144,7 @@ def test_validate():
         @classmethod
         def format2(cls):
             return "hello2"
+
     # no exception for callable methods
     tup = MyTuple(['--abc', 'abc'])
     assert tup.abc == 'abc'
@@ -141,7 +153,6 @@ def test_validate():
 
 
 def test_validate_fields():
-
     class DanglingParamOverride(NamedTuple):
         _a_str = "abc"
 
@@ -178,17 +189,18 @@ def test_argv():
 
 
 def test_primitive_addon():
-
     class IntHandlerAddon(PrimitiveHandlerAddon):
         @staticmethod
         def build_primitive(arg_type, kwargs):
             if arg_type is int:
                 kwargs.type = lambda s: int(s) ** 2
+
         handled_types = [int]
 
     class IntTypeHandler(TypeHandler):
         def _build_common(self, kwargs, field_meta):
             kwargs.help = '(int, squared)'
+
         handled_types = [int]
 
     @custom_arg_suite(primitive_handler_addons=[IntHandlerAddon], type_handlers=[IntTypeHandler])
@@ -204,6 +216,7 @@ def test_primitive_addon():
 def test_unsupported_types():
     class MyTuple(NamedTuple):
         a: List[List[int]]
+
     pytest.raises(SmartArgError, arg_suite, MyTuple)
 
 
@@ -212,6 +225,7 @@ def test_nested():
     class MyTup(NamedTuple):
         def __late_init__(self):
             return self._replace(b_int=10) if self.b_int is LateInit else self
+
         b_int: int = LateInit  # if a_int is not in the argument, __late_init__ will initialize it
 
     @arg_suite
@@ -231,8 +245,12 @@ def test_nested():
     class NotDecoratedWithPost(NamedTuple):
         def __validate__(self): pass
 
-    pytest.raises(SmartArgError, arg_suite, NamedTuple('Nested', nested=NotDecoratedWithPost))
-    arg_suite(NamedTuple('Nested', nested=NamedTuple('NotDecorated')))  # should not raise
+    class Nested(NamedTuple):
+        nested: NotDecoratedWithPost
+
+    pytest.raises(SmartArgError, arg_suite, Nested)
+    del NotDecoratedWithPost.__validate__
+    arg_suite(Nested)  # should not raise
 
 
 def test_cli_execution():
